//////////////////////////////////////////////////////
/// Kello HW v3.1 FW v1.3 JAKA 5.9.2014
//////////////////////////////////////////////////////
///  Pinout: I2C SDA  = B4
///			 I2C SCL  = B6


#case
#zero_ram
#include <18F14K50.h>
#include <stdlib.h>
#include <ctype.h>
#fuses CPUDIV1,INTRC_IO,NOPLLEN,PUT,BROWNOUT,BORV30,NOWDT,WDT128,NOHFOFST,NOMCLR,NOPROTECT,NOLVP,NODEBUG
#use delay(clock=8000000)
#use i2c(master,sda=PIN_B4,scl=PIN_B6,fast=70000,force_sw) 	// SAA1064 can reliably handle only 70kbps I2C
#include "saa1064_kellov3.h"
#include "ds3231.c"
#define bool short
#define bit  short
// #define STATUS 0x3	// for PIC16
#define STATUS 0xFD8	// for PIC18
#define C 0 // Carry is bit 0 in status...
#define IRin 31765  // pin C5...

#define BIT_TIME 11  // Max bit time in 100us units.
#define PULSE_MAX 7 // Max pulse width in 100us units

unsigned int16 intcounter = 0;
unsigned int16 actcycles = 0;		// How many timer cycles since last keypress,
									// is the repetition still valid? 
unsigned int8 repeats=0;
unsigned int8 IR_SHIFT1;
unsigned int8 IR_SHIFT2;
unsigned int8 IR_SHIFT3;
unsigned int8 IR_SHIFT4;
unsigned int8 IR_state = 0;
unsigned int8 MENU_state;
unsigned int8 DIGIT1;
unsigned int8 DIGIT2;
unsigned int8 DIGIT3;
unsigned int8 DIGIT4;
unsigned int8 BIT_TIMER;
unsigned int8 tunnit = 0;
unsigned int8 minuutit = 0;
unsigned int8 sekunnit = 0;
unsigned int8 paiva = 12;
unsigned int8 kuukausi = 11;
unsigned int8 vuosi = 11;
unsigned int8 current;
bool ir_received = FALSE;
bool updatedisplay = TRUE;
bool localcounter = FALSE;
unsigned int8 remote_button = 0;
unsigned int8 degrees = 0;
unsigned int8 fragments = 0;
unsigned int8 joyprev = 0;
unsigned int8 joycurr = 0;

#byte UCON = 0xF64
#bit USBEN = UCON.3

bool ir_machine(void);	// The ir reveiver state machine
bool menu_machine(void);// The menu state machine
void code_to_button(void); // Decodes reveived code to a button
void adjust_current(void); // Adds 3mA, up to 15mA
void save_date(void);	// Saves date to clock chip, corrects for invalid date
void save_time(void);	// Saves time to clock chip
void read_jstk(void);	// Reads joystick switch

void main (void) {
	output_low(PIN_C5);
	output_high(PIN_C7);
	port_a_pullups(0xff);
	port_b_pullups(0xff);

	USBEN=0;	// disable USB
	output_high(PIN_B5);		// Pullup for SCL

	send_saa1064_raw(LED_ADDR1,0,0b00000000); // added as extra i2c activity after power-up
	send_saa1064_raw(LED_ADDR1,0,0b00000000); // to work around time corruption bug
	send_saa1064_raw(LED_ADDR1,0,0b00000000); // 29.7.2014 jka
	send_saa1064_raw(LED_ADDR1,0,0b00000000);

	if (ds3231_check_por()) {	// Check if booted without battery
		set_saa1064_current(current);	// for some reason must have other i2c activity between 2 get bytes(?)
		ds3231_set_byte(0x00, 0x08);
		set_saa1064_current(current);	// for some reason must have other i2c activity between 2 get bytes(?)
		current = 3;
		MENU_state = 1;
		ds3231_set_byte(current, 0x01);
		set_saa1064_current(current);	// for some reason must have other i2c activity between 2 get bytes(?)
		ds3231_set_byte(MENU_state, 0x02);
	}
	else {
		set_saa1064_current(current);	// for some reason must have other i2c activity between 2 get bytes(?)
		ds3231_get_byte(current, 0x01);
		set_saa1064_current(current);	// for some reason must have other i2c activity between 2 get bytes(?)
		ds3231_get_byte(MENU_state, 0x02);
	}	

	setup_timer_2(T2_DIV_BY_1,200,1); // set timer2 to 100us (prescale 1, postscale 1, period 200) // 8 MHz
	
	enable_interrupts(GLOBAL);
	enable_interrupts(INT_TIMER2);
	
	while(1) {
		if (ir_received)
			code_to_button();
		if (remote_button==1)
			adjust_current();
		menu_machine();
	}	
}

#int_timer2
void timer2isr(void) {
	
	if (ir_machine())
		ir_received = TRUE;		
	
	read_jstk();	
	
	if (localcounter) {				// Now used only when adjusting time..
		intcounter++;
	   
	   if (intcounter == 10000) {
	      intcounter = 0;
	      sekunnit++;
	      if (sekunnit == 60) {
	         sekunnit = 0;
	         minuutit++;
	         if (minuutit == 60) {
	            minuutit = 0;
	            tunnit++;
	            if (tunnit == 24) {
	               tunnit = 0;
	            }      
	         }
	      }
	   }
 	}  
}


/// Kinkkikake:
/// POWER:  3f c0 fd 02
/// CH DN:  c5 3a fd 02
/// CH UP:  3d c2 fd 02
/// MUTE:   77 88 fd 02
/// VOL DN: df 20 fd 02
/// VOL UP: b5 4a fd 02
/// AV/TV:  a5 5a fd 02

void adjust_current(void) {
	current = current + 3;
	if (current == 18)
		current = 3;
	set_saa1064_current(current);
	ds3231_set_byte(current, 0x01);
}	

void save_date(void) {
	if ((kuukausi == 4 || kuukausi == 6 || kuukausi == 9 || kuukausi == 11)&& paiva > 30)
		paiva = 30;
	if (kuukausi == 2 && paiva > 28)
		paiva = 28;
		
	ds3231_set_date(paiva, kuukausi, vuosi, 1);
}		

void save_time(void) {
	ds3231_set_time(tunnit, minuutit, sekunnit);
	localcounter=FALSE;
}		

void read_jstk(void) {
		joyprev=joycurr;
		joycurr=0;
		if (!input(PIN_C0))	// up
			joycurr=2;
		if (!input(PIN_B7))	// dn
			joycurr=3;
		if (!input(PIN_A4))	// act
			joycurr=4;
		if (!input(PIN_A3))	// left
			joycurr=5;
		if (!input(PIN_A5))	// right
			joycurr=6;
		if (joycurr>0 && joyprev==0)
			remote_button=joycurr;
}		

void code_to_button(void) {
	remote_button = 0;
	if (DIGIT3==0xfd && DIGIT4==0x02) { // Kinkki remote..
				if (DIGIT1==0x3f && DIGIT2==0xc0) { // Power
					remote_button = 1;
				}
				else if (DIGIT1==0x3d && DIGIT2==0xc2) { // Ch Up
					remote_button = 2;
				}
				else if (DIGIT1==0xc5 && DIGIT2==0x3a) { // Ch Down
					remote_button = 3;
				}
				else if (DIGIT1==0x77 && DIGIT2==0x88) { // Mute
					remote_button = 4;
				}
				else if (DIGIT1==0xdf && DIGIT2==0x20) { // Vol Down
					remote_button = 5;
				}
				else if (DIGIT1==0xb5 && DIGIT2==0x4a) { // Vol Up
					remote_button = 6;
				}
				else if (DIGIT1==0xa5 && DIGIT2==0x5a) { // AV/TV
					remote_button = 7;
				}
	}
	ir_received=FALSE;
}	

bool menu_machine(void) {
	switch (MENU_state) {
		case 1:
			ds3231_get_time(tunnit, minuutit, sekunnit);
			if (updatedisplay) {				// shows time
						send_saa1064_num(LED_ADDR0,LED_CONTROL,tunnit,0);
						send_saa1064_num(LED_ADDR3,LED_CONTROL,minuutit,2);
						send_saa1064_num(LED_ADDR1,LED_CONTROL,sekunnit,2);
			}
			if (remote_button == 4)
				MENU_state = 10;
			remote_button = 0;
			return TRUE;
		case 2:
			ds3231_get_date(paiva, kuukausi, vuosi, degrees);
			if (updatedisplay) {				// shows date
						send_saa1064_num(LED_ADDR0,LED_CONTROL,paiva,1);
						send_saa1064_num(LED_ADDR3,LED_CONTROL,kuukausi,1);
						send_saa1064_num(LED_ADDR1,LED_CONTROL,vuosi,0);
			}
			if (remote_button == 4)
				MENU_state = 11;
			remote_button = 0;
			return TRUE;
		case 3:
			ds3231_get_temp(degrees, fragments);
			if (updatedisplay) {				// shows temp
						send_saa1064_num(LED_ADDR0,LED_CONTROL,degrees,1);
						send_saa1064_num(LED_ADDR3,LED_CONTROL,fragments,0);
						send_saa1064_raw(LED_ADDR1,0,0b00001111); // 
						send_saa1064_raw(LED_ADDR1,1,0b11001010); // C
			}
			if (remote_button == 4)
				MENU_state = 12;
			remote_button = 0;
			return TRUE;
		case 4:
			ds3231_get_time(tunnit, minuutit, sekunnit);
			if((sekunnit%15)==0 || ((sekunnit-1)%15)==0) {
				ds3231_get_temp(degrees, fragments);	
				if (updatedisplay) {				// shows toggling time, date and temp
					send_saa1064_num(LED_ADDR0,LED_CONTROL,degrees,1);
					send_saa1064_num(LED_ADDR3,LED_CONTROL,fragments,0);
					send_saa1064_raw(LED_ADDR1,0,0b00001111); // 
					send_saa1064_raw(LED_ADDR1,1,0b11001010); // C
				}
			}
			else if (((sekunnit-2)%15)==0 || ((sekunnit-3)%15)==0) {	
				ds3231_get_date(paiva, kuukausi, vuosi, degrees);
				if (updatedisplay) {				// shows date
					send_saa1064_num(LED_ADDR0,LED_CONTROL,paiva,1);
					send_saa1064_num(LED_ADDR3,LED_CONTROL,kuukausi,1);
					send_saa1064_num(LED_ADDR1,LED_CONTROL,vuosi,0);
				}
			} else {
				if (updatedisplay) {				// shows time
					send_saa1064_num(LED_ADDR0,LED_CONTROL,tunnit,0);
					send_saa1064_num(LED_ADDR3,LED_CONTROL,minuutit,2);
					send_saa1064_num(LED_ADDR1,LED_CONTROL,sekunnit,2);
				}
			}
				
			if (remote_button == 4)
				MENU_state = 13;
			remote_button = 0;
			return TRUE;
		case 10:
			if (updatedisplay) {				// shows 'hour'
						send_saa1064_raw(LED_ADDR0,0,0);
						send_saa1064_raw(LED_ADDR0,1,0);
						send_saa1064_raw(LED_ADDR3,0,0b10101100); // h
						send_saa1064_raw(LED_ADDR3,1,0b11100001); // o
						send_saa1064_raw(LED_ADDR1,0,0b11100000); // u
						send_saa1064_raw(LED_ADDR1,1,0b10000001); // r
			}
			if (remote_button == 4) {
				MENU_state = 1;
				ds3231_set_byte(MENU_state, 0x02);
			}	
			if (remote_button == 6)
				MENU_state = 11;
			if (remote_button == 5)
				MENU_state = 14;
			remote_button = 0;
			return TRUE;
		case 11:
			if (updatedisplay) {				// shows 'date'
						send_saa1064_raw(LED_ADDR0,0,0);
						send_saa1064_raw(LED_ADDR0,1,0);
						send_saa1064_raw(LED_ADDR3,0,0b11101001); // d
						send_saa1064_raw(LED_ADDR3,1,0b10111011); // A
						send_saa1064_raw(LED_ADDR1,0,0b01101100); // t
						send_saa1064_raw(LED_ADDR1,1,0b11001011); // E
			}
			if (remote_button == 4) {
				MENU_state = 2;
				ds3231_set_byte(MENU_state, 0x02);
			}		
			if (remote_button == 6)
				MENU_state = 12;
			if (remote_button == 5)
				MENU_state = 10;
			remote_button = 0;
			return TRUE;
		case 12:
			if (updatedisplay) {				// shows 'dEGrEE'
						send_saa1064_raw(LED_ADDR0,0,0b11101001); // d
						send_saa1064_raw(LED_ADDR0,1,0b11001011); // E
						send_saa1064_raw(LED_ADDR3,0,0b11100110); // G
						send_saa1064_raw(LED_ADDR3,1,0b10000001); // r
						send_saa1064_raw(LED_ADDR1,0,0b01101110); // E
						send_saa1064_raw(LED_ADDR1,1,0b11001011); // E
			}
			if (remote_button == 4) {
				MENU_state = 3;
				ds3231_set_byte(MENU_state, 0x02);
			}		
			if (remote_button == 6)
				MENU_state = 13;
			if (remote_button == 5)
				MENU_state = 11;
			remote_button = 0;
			return TRUE;
		case 13:
			if (updatedisplay) {				// shows 'toGGLE'
						send_saa1064_raw(LED_ADDR0,0,0b01101100); // t
						send_saa1064_raw(LED_ADDR0,1,0b11100001); // o
						send_saa1064_raw(LED_ADDR3,0,0b11100110); // G
						send_saa1064_raw(LED_ADDR3,1,0b11101010); // G
						send_saa1064_raw(LED_ADDR1,0,0b01100100); // L
						send_saa1064_raw(LED_ADDR1,1,0b11001011); // E
			}
			if (remote_button == 4) {
				MENU_state = 4;
				ds3231_set_byte(MENU_state, 0x02);
			}		
			if (remote_button == 6)
				MENU_state = 14;
			if (remote_button == 5)
				MENU_state = 12;
			remote_button = 0;
			return TRUE;			
		case 14:
			if (updatedisplay) {				// shows 'set'
						send_saa1064_raw(LED_ADDR0,0,0);
						send_saa1064_raw(LED_ADDR0,1,0);
						send_saa1064_raw(LED_ADDR3,0,0);    // 
						send_saa1064_raw(LED_ADDR3,1,0b01101011); // S
						send_saa1064_raw(LED_ADDR1,0,0b01101110); // E
						send_saa1064_raw(LED_ADDR1,1,0b11000011); // t
			}
			if (remote_button == 4)
				MENU_state = 20;
			if (remote_button == 6)
				MENU_state = 10;
			if (remote_button == 5)
				MENU_state = 13;
			remote_button = 0;
			return TRUE;			
		case 20:
			if (updatedisplay) {				// shows 'sethou'
						send_saa1064_raw(LED_ADDR0,0,0b11001110); // S
						send_saa1064_raw(LED_ADDR0,1,0b11001011); // E
						send_saa1064_raw(LED_ADDR3,0,0b01101100); // t
						send_saa1064_raw(LED_ADDR3,1,0b10100011); // h
						send_saa1064_raw(LED_ADDR1,0,0b11101000); // o
						send_saa1064_raw(LED_ADDR1,1,0b11100000); // u
			}
			if (remote_button == 4) {
				ds3231_get_time(tunnit, minuutit, sekunnit);
				MENU_state = 21;
			}	
			if (remote_button == 6)
				MENU_state = 30;
			if (remote_button == 5)
				MENU_state = 14;
			remote_button = 0;
			return TRUE;
		case 21:
			localcounter = TRUE;
			if (updatedisplay) {				// adjust hour
						send_saa1064_num(LED_ADDR0,0b01000110,tunnit,0);
						send_saa1064_num(LED_ADDR3,0b00010110,minuutit,2);
						send_saa1064_num(LED_ADDR1,0b00010110,sekunnit,2);
			}
			if (remote_button == 4) {
				MENU_state = 20;
				set_saa1064_current(current);
				save_time();
			}	
			if (remote_button == 6)
				MENU_state = 22;
			if (remote_button == 5)
				MENU_state = 23;
				
			if (remote_button == 2) {	// UP
				if (tunnit<23)
					tunnit++;
				else if (tunnit==23)
					tunnit=0;
			}
		
			if (remote_button == 3) {	// DN
				if (tunnit>0)
					tunnit--;
				else if (tunnit==0)
					tunnit=23;
			}	
			remote_button = 0;
			return TRUE;
		case 22:
			if (updatedisplay) {				// adjust minutes
						send_saa1064_num(LED_ADDR0,0b00010110,tunnit,0);
						send_saa1064_num(LED_ADDR3,0b01000110,minuutit,2);
						send_saa1064_num(LED_ADDR1,0b00010110,sekunnit,2);
			}
			if (remote_button == 4) {
				MENU_state = 20;
				set_saa1064_current(current);
				save_time();
			}	
			if (remote_button == 6)
				MENU_state = 23;
			if (remote_button == 5)
				MENU_state = 21;
			
			if (remote_button == 2) {	// UP
				if (minuutit<59)
					minuutit++;
				else if (minuutit==59)
					minuutit=0;
			}
			
			if (remote_button == 3) {	// DN
				if (minuutit>0)
					minuutit--;
				else if (minuutit==0)
					minuutit=59;
			}	
			
			remote_button = 0;
			return TRUE;
		case 23:
			if (updatedisplay) {				// zero seconds
						send_saa1064_num(LED_ADDR0,0b00010110,tunnit,0);
						send_saa1064_num(LED_ADDR3,0b00010110,minuutit,2);
						send_saa1064_num(LED_ADDR1,0b01000110,sekunnit,2);
			}
			if (remote_button == 4) {
				MENU_state = 20;
				set_saa1064_current(current);
				save_time();
			}	
			if (remote_button == 6)
				MENU_state = 21;
			if (remote_button == 5)
				MENU_state = 22;
			
			if (remote_button == 2 || remote_button == 3) {	// UP or DN = zero
				if (sekunnit>30) {
					if (minuutit<59)
						minuutit++;
					else if (minuutit==59) {
						minuutit=0;
						if (tunnit<23)
							tunnit++;
						else if (tunnit==23)
							tunnit=0;
					}		
				}
				sekunnit=0;
				intcounter=0;
			}
			remote_button = 0;
			return TRUE;
		case 30:
			if (updatedisplay) {				// shows 'setdat'
						send_saa1064_raw(LED_ADDR0,0,0b11001110); // S
						send_saa1064_raw(LED_ADDR0,1,0b11001011); // E
						send_saa1064_raw(LED_ADDR3,0,0b01101100); // t
						send_saa1064_raw(LED_ADDR3,1,0b11110001); // d
						send_saa1064_raw(LED_ADDR1,0,0b10101111); // A
						send_saa1064_raw(LED_ADDR1,1,0b11000011); // t
			}
			if (remote_button == 4) {
				ds3231_get_date(paiva, kuukausi, vuosi, degrees);
				MENU_state = 31;
			}	
			if (remote_button == 6)
				MENU_state = 40;
			if (remote_button == 5)
				MENU_state = 20;
			remote_button = 0;
			return TRUE;			
		case 31:
			if (updatedisplay) {				// adjust day
						send_saa1064_num(LED_ADDR0,0b01000110,paiva,1);
						send_saa1064_num(LED_ADDR3,0b00010110,kuukausi,1);
						send_saa1064_num(LED_ADDR1,0b00010110,vuosi,0);
			}
			if (remote_button == 4) {
				MENU_state = 30;
				set_saa1064_current(current);
				save_date();
			}	
			if (remote_button == 6)
				MENU_state = 32;
			if (remote_button == 5)
				MENU_state = 33;
				
			if (remote_button == 2) {	// UP
				if (paiva<31)
					paiva++;
				else if (paiva==31)
					paiva=1;
			}
		
			if (remote_button == 3) {	// DN
				if (paiva>1)
					paiva--;
				else if (paiva==1)
					paiva=31;
			}	
			remote_button = 0;
			return TRUE;
		case 32:
			if (updatedisplay) {				// adjust month
						send_saa1064_num(LED_ADDR0,0b00010110,paiva,1);
						send_saa1064_num(LED_ADDR3,0b01000110,kuukausi,1);
						send_saa1064_num(LED_ADDR1,0b00010110,vuosi,0);
			}
			if (remote_button == 4) {
				MENU_state = 30;
				set_saa1064_current(current);
				save_date();
			}	
			if (remote_button == 6)
				MENU_state = 33;
			if (remote_button == 5)
				MENU_state = 31;
			
			if (remote_button == 2) {	// UP
				if (kuukausi<12)
					kuukausi++;
				else if (kuukausi==12)
					kuukausi=1;
			}
			
			if (remote_button == 3) {	// DN
				if (kuukausi>1)
					kuukausi--;
				else if (kuukausi==1)
					kuukausi=12;
			}	
			
			remote_button = 0;
			return TRUE;
		case 33:
			if (updatedisplay) {				// adjust year
						send_saa1064_num(LED_ADDR0,0b00010110,paiva,1);
						send_saa1064_num(LED_ADDR3,0b00010110,kuukausi,1);
						send_saa1064_num(LED_ADDR1,0b01000110,vuosi,0);
			}
			if (remote_button == 4) {
				MENU_state = 30;
				set_saa1064_current(current);
				save_date();
			}	
			if (remote_button == 6)
				MENU_state = 31;
			if (remote_button == 5)
				MENU_state = 32;
			
			if (remote_button == 2) {	// UP
				if (vuosi<99)
					vuosi++;
				else if (vuosi==99)
					vuosi=0;
			}
			
			if (remote_button == 3) {	// DN
				if (vuosi>0)
					vuosi--;
				else if (vuosi==0)
					vuosi=99;
			}		
				
			remote_button = 0;
			return TRUE;
		case 40:
			if (updatedisplay) {				// shows 'setbri'
						send_saa1064_raw(LED_ADDR0,0,0b11001110); // S
						send_saa1064_raw(LED_ADDR0,1,0b11001011); // E
						send_saa1064_raw(LED_ADDR3,0,0b01101100); // t
						send_saa1064_raw(LED_ADDR3,1,0b11100011); // b
						send_saa1064_raw(LED_ADDR1,0,0b00101000); // r
						send_saa1064_raw(LED_ADDR1,1,0b00100000); // i
			}
			if (remote_button == 4) {
				ds3231_get_time(tunnit, minuutit, sekunnit);
				MENU_state = 41;
			}	
			if (remote_button == 6)
				MENU_state = 14;
			if (remote_button == 5)
				MENU_state = 30;
			remote_button = 0;
			return TRUE;
		case 41:
			if (updatedisplay) {				// adjust current (brightness)
						send_saa1064_raw(LED_ADDR0,0,0b01101000); // c
						send_saa1064_raw(LED_ADDR0,1,0b11100000); // u
						send_saa1064_raw(LED_ADDR3,0,0b00101000); // r
						send_saa1064_raw(LED_ADDR3,1,0b00000000); //  
						send_saa1064_num(LED_ADDR1,LED_CONTROL,current,0);
			}
			if (remote_button == 4) {
				MENU_state = 40;
				ds3231_set_byte(current, 0x01);
			}	
				
			if (remote_button == 2) {	// UP
				if (current<15)
					current = current + 3;
				set_saa1064_current(current);
			}
		
			if (remote_button == 3) {	// DN
				if (current>3)
					current = current - 3;
				set_saa1064_current(current);
			}	
			remote_button = 0;
			return TRUE;

			default: return FALSE;
	}
}		
		
bool ir_machine(void) {
	switch (IR_state) {
		case 0: if (ir_received)	// Do not receive new ir before previous 
					return FALSE;	// has been procssed
				if (actcycles!=0)
					actcycles--;
				if (input(IRin)) // input still high?
					return FALSE;		 // yes, nothing to do here

				IR_SHIFT1=1;
				IR_SHIFT2=0;
				IR_SHIFT3=0;
				IR_SHIFT4=0;
				BIT_TIMER=0;    // Clear bit timer
				IR_state=1;		// Next state is 1
				return FALSE;
		case 1: BIT_TIMER++;	// increment bit timer
				if (!(input(IRin))) { // input still low?
					if (BIT_TIMER>99) { // Longer than 9ms + 10% low?
						IR_state=8;	 // Not good! goto error state
						return FALSE;
					}
					else
						return FALSE;	 // OK, we'll wait..
				}
				else {
					if (BIT_TIMER>81) {  // At least 9ms - 10% low? (8100/100)
						IR_state=2;		  // next state is 2
						BIT_TIMER=0;	  // Clear BIT_TIMER
						return FALSE;
					}
					else {
						IR_state=0;	  // too short, go back to state 0
						return FALSE;
					}
				}
		// *** STATE 2, PRE-PULSE DETECTED, NOW MEASURE GAP TIME ***
		case 2: BIT_TIMER++;
				if (input(IRin)) {			// Input still high?
					if (BIT_TIMER>50) {		// Longer than 4.5ms + 10% high?
						IR_state=0;		// Yes, not good! Start from state 0.
						return FALSE;
					}
					else
						return FALSE;			// OK, not yet. We'll wait...
				}
				else {						// Input is low.. 
					if (BIT_TIMER>34) {		// Let's see if the gap was long (>3.35ms)
						IR_state=4;			// It was long, go to state 4 and decode message
						BIT_TIMER=0;
						return FALSE;
					}
					else {
						IR_state=3;			// It was a short one! It's a repetition!
						BIT_TIMER=0;
						return FALSE;
					}
				}
		// *** STATE 3, REPETITION DETECTED, LET'S SEE IF DISPLAY IS STILL VALID ***
		case 3: IR_state=8;					// Next state will be 8 anyway, so let's set it now
				if (actcycles!=0) {   // Was there less than 0.5 sec since last key/repeat?
					actcycles=5000;
					if (repeats<5)
						repeats++;
					if (repeats==0 || repeats==5)
						return TRUE;
					else
						return FALSE;
				}
				else
					return FALSE;
		// *** STATE 4, INPUT IS LOW, MUST BE A PULSE THEN ***
		case 4: BIT_TIMER++;
				if (!(input(IRin))) {  // Input still low?
					if (BIT_TIMER>PULSE_MAX) { // See if max pulse width reached
						IR_state=8; 			// Max width reached, goto error 8 wait it gets high
						return FALSE;
					}
					else {
						return FALSE;			// Not yet!
					}
				}
				else {
					IR_state=5;		// Input is now high
					return FALSE;	// Next stop is state 5	
				}
		// *** STATE 5, WAIT FOR PULSE TO GO LOW AGAIN, THEN MEASURE INTERVAL ***
		case 5: BIT_TIMER++;
				if (BIT_TIMER < BIT_TIME*3) { // Should we keep waiting?
					if (input(IRin)) { // Yes. Is the input still high?
						return FALSE;  // It is - we'll wait..
					}
					else {

						
						#asm ASIS
						MOVLW   238			// BIT_TIME/2+BIT_TIME*-1  ;Use 1.5 bit time as threshold
		                ADDWF   BIT_TIMER,W     // Carry is 1 if more than 1.5 times

                		RLCF     IR_SHIFT1,F  	// Roll carry into result
                		RLCF     IR_SHIFT2,F
                		RLCF     IR_SHIFT3,F
                		RLCF     IR_SHIFT4,F
                		
                		CLRF    BIT_TIMER       // Restart bit timer
		                MOVLW   4	  	 	 	// Return to state 4 if not done yet
        		        MOVWF   IR_state

                		BTFSS   STATUS,C    	// See if we're done
               			GOTO	asmend          // Not done yet, continue with state 4

                		MOVLW   6				// Message received, but wait 3ms longer
                		MOVWF   IR_state        // to see if nothing follows.
                		CLRF    BIT_TIMER
                		asmend:
                		#endasm
                		
						return FALSE;
					}
				}
				else {  // We've waited 3 bit times already!
					IR_state = 0; // Error, lets start from beginning..
					output_toggle(PIN_C7);
					return FALSE;
				}
		// *** STATE 6, RECEIVED MESSAGE, WAIT 3MS TO ENSURE IT WAS NEC ***
		case 6:	BIT_TIMER++;
				if (!(input(IRin))) { // Input still low?
					if (BIT_TIMER > PULSE_MAX) { // See if max pulse width reached
						IR_state=8; // Goto to error state 8
						return FALSE;
					}
					else { 			
						return FALSE;
					}
				}
				else {
					IR_state=7;		// Input is now high
					return FALSE;	// Continue with 3ms delay..
				}			
		// *** STATE 7, INPUT MUST REMAIN HIGH FOR AT LEAST 3MS ***
		case 7: BIT_TIMER++;
				if (!(input(IRin))) { // Has the input gone low now?
					IR_state=8;		// Yes! Not good, goto error state 10
					return FALSE;
				}
				else {
					if (BIT_TIMER > BIT_TIME*3) {
						#asm ASIS
			     		MOVF    IR_SHIFT1,W      // Work from left to right
            		    MOVWF   DIGIT1
				        MOVF    IR_SHIFT2,W      // Do the same with 2nd digit
               			MOVWF   DIGIT2
						MOVF    IR_SHIFT3,W      // And with the 3d digit
                		MOVWF   DIGIT3
						MOVF    IR_SHIFT4,W      // And finally with the last digit
                		MOVWF   DIGIT4
		        		
						#endasm
						IR_state=0;		// We're done! Let's start from beginning
						actcycles=5000; // Set active cycles counter to 10000 (0.5 sec)
						repeats=0;
						return TRUE;	// Return TRUE, we have a valid code!
					}
					else {
						return FALSE;	// Keep on waiting..
					}
				}
		// *** ERROR OR REPETITION ***
		case 8:	if (input(IRin)) { // Wait for input to return high..
					IR_state=0;		// It is!
					return FALSE;
				}
				else {
					return FALSE;	// not yet..
				}
		default: return FALSE; // Should not get here but anyway..
	}
}			
